proc datasets lib = work kill; run;
option linesize = 200 pagesize = 5000 mprint;
dm 'log; clear; output; clear;';

%let execpath = " ";
%let Path = " ";
%macro setexecpath;
  %let execpath = %sysfunc(getoption(sysin));
  %if %length(&execpath) = 0 %then
    %let execpath = %sysget(sas_execfilepath);
  data _null_;
    do i = length("&execpath") to 1 by -1;
      if substr("&execpath", i, 1) = "\" then do;
        call symput("Path", substr("&execpath", 1, i));
        stop;
      end;
    end;
  run;
%mend setexecpath;
%setexecpath;
libname Out "&Path";

data D1;
  length GroupC GroupC2 $20.;
  call streaminit(32789238);
  tc = 12;  /* withdrawal time */
  h1 = 0.100;  n1 = 50; /* h1: hazard of group 1 */
  h2 = 0.075;  n2 = 50; /* h2: hazard of group 2 */
  h3 = 0.050;  n3 = 50; /* h3: hazard of group 3 */
  /* E(T)=1/h */
  array h[*] h1-h3; array n[*] n1-n3;
  do Group = 1 to 3;
    do i = 1 to n[group];
      T = rand('exponential') / h[group]; Censor = 0;
      if T > tc then do; T = tc; Censor = 1; end;
      if rand('Uniform') > 0.9 then do; Censor = 1; end;
/* lost to follow-up (random) */
      if Group = 1 then do; GroupC = '1: high-risk'; GroupC2 = 'high-risk'; end;
      if Group = 2 then do; GroupC = '2: middle-risk'; GroupC2 = 'middle-risk'; end;
      if Group = 3 then do; GroupC = '3: low-risk'; GroupC2 = 'low-risk'; end;
     output;
    end;
  end;
  label T = "Months after entry";
  keep T Censor Group GroupC GroupC2;
run;


proc template;
  delete Stat.Lifetest.Graphics.ProductLimitSurvival;
run;
/*ods path sashelp.tmplmst(read);*/
/*proc datasets library=sasuser;*/
/*  delete templat(memtype=itemstor);*/
/*run;*/
/*ods path sasuser.templat(update) sashelp.tmplmst(read);*/
ods listing gpath = "&Path." style = Statistical sge = on;
ods graphics on /
  antialias = on border = off scale = on
  imagename = "Lifetest_ods"
  width = 6.33333333 in height = 4 in;
proc lifetest data = D1 plots=(survival(atrisk=(0 to 12.5 by 2.5) test cl));
  time T * Censor(1);
  strata GroupC2;
run;
ods graphics off;
ods listing close; ods listing;

proc template;
define style Styles.MyStatistical;
parent = styles.Statistical;
style GraphFonts /
  'GraphTitleFont'=("Times New Roman",24pt, bold)
  'GraphFootnoteFont'=("Times New Roman",24pt, italic)
  'GraphLabelFont'=("Times New Roman",24pt)
  'GraphValueFont'=("Times New Roman", 24pt)
  'GraphDataFont'=("Times New Roman", 24pt)
  'GraphUnicodeFont'=(" <MTsans-serif-unicode> ", 24pt)
  'GraphAnnoFont'=("Times New Roman", 24pt);
end;
proc template;
define statgraph Stat.Lifetest.Graphics.ProductLimitSurvival;
  dynamic NStrata xName plotAtRisk plotCensored plotCL plotHW plotEP labelCL labelHW labelEP
      maxTime StratumID classAtRisk plotBand plotTest GroupName yMin Transparency SecondTitle
      TestName pValue;
  BeginGraph;
    if (NSTRATA=1)
      layout overlay / xaxisopts=(shortlabel=XNAME offsetmin=.05 linearopts=(viewmax=MAXTIME))
          yaxisopts=(label="Survival Probability" shortlabel="Survival Probability"
          linearopts=(viewmin=0 viewmax=1 tickvaluelist=(0 .2 .4 .6 .8 1.0)));
        if (PLOTHW=1 AND PLOTEP=0)
          bandplot LimitUpper=HW_UCL LimitLower=HW_LCL x=TIME / modelname="Survival"
              fillattrs=GRAPHCONFIDENCE name="HW" legendlabel=LABELHW;
        endif;
        if (PLOTHW=0 AND PLOTEP=1)
          bandplot LimitUpper=EP_UCL LimitLower=EP_LCL x=TIME / modelname="Survival"
              fillattrs=GRAPHCONFIDENCE name="EP" legendlabel=LABELEP;
        endif;
        if (PLOTHW=1 AND PLOTEP=1)
          bandplot LimitUpper=HW_UCL LimitLower=HW_LCL x=TIME / modelname="Survival"
              fillattrs=GRAPHDATA1 datatransparency=.55 name="HW" legendlabel=LABELHW;
          bandplot LimitUpper=EP_UCL LimitLower=EP_LCL x=TIME / modelname="Survival"
              fillattrs=GRAPHDATA2 datatransparency=.55 name="EP" legendlabel=LABELEP;
        endif;
        if (PLOTCL=1)
          if (PLOTHW=1 OR PLOTEP=1)
            bandplot LimitUpper=SDF_UCL LimitLower=SDF_LCL x=TIME / modelname="Survival"
                display=(outline) outlineattrs=GRAPHPREDICTIONLIMITS name="CL" legendlabel=LABELCL;
          else
            bandplot LimitUpper=SDF_UCL LimitLower=SDF_LCL x=TIME / modelname="Survival"
                fillattrs=GRAPHCONFIDENCE name="CL" legendlabel=LABELCL;
          endif;
        endif;
        stepplot y=SURVIVAL x=TIME / name="Survival" rolename=(_tip1=ATRISK _tip2=EVENT)
            tip=(y x Time _tip1 _tip2) legendlabel="Survival Probability" LINEATTRS=(thickness=10px);
        if (PLOTCENSORED=1)
          scatterplot y=CENSORED x=TIME / markerattrs=(symbol=CIRCLEFILLED size=8pt)
              name="Censored" legendlabel="Censored";
        endif;
        if (PLOTCL=1 OR PLOTHW=1 OR PLOTEP=1)
          discretelegend "Censored" "CL" "HW" "EP" / location=outside halign=center;
        else
          if (PLOTCENSORED=1)
            discretelegend "Censored" / location=inside autoalign=(topright bottomleft);
          endif;
        endif;
        if (PLOTATRISK=1)
          innermargin / align=bottom;
            blockplot x=TATRISK block=ATRISK / repeatedvalues=true display=(values) valuehalign=
                start valuefitpolicy=truncate labelposition=left labelattrs=GRAPHVALUETEXT
                valueattrs=GRAPHDATATEXT (size=20pt) includemissingclass=false;
          endinnermargin;
        endif;
      endlayout;
    else
      layout overlay / xaxisopts=(shortlabel=XNAME offsetmin=.05 linearopts=(viewmax=MAXTIME))
          yaxisopts=(label="Survival Probability" shortlabel="Survival Probability"
          linearopts=(viewmin=0 viewmax=1 tickvaluelist=(0 .2 .4 .6 .8 1.0)));
        if (PLOTHW)
          bandplot LimitUpper=HW_UCL LimitLower=HW_LCL x=TIME / group=STRATUM
              index=STRATUMNUM modelname="Survival" datatransparency=Transparency;
        endif;
        if (PLOTEP)
          bandplot LimitUpper=EP_UCL LimitLower=EP_LCL x=TIME / group=STRATUM
              index=STRATUMNUM modelname="Survival" datatransparency=Transparency;
        endif;
        if (PLOTCL)
          if (PLOTBAND)
            bandplot LimitUpper=SDF_UCL LimitLower=SDF_LCL x=TIME / group=STRATUM index=
              STRATUMNUM modelname="Survival" display=(outline);
          else
            bandplot LimitUpper=SDF_UCL LimitLower=SDF_LCL x=TIME / group=STRATUM index=
              STRATUMNUM modelname="Survival" datatransparency=Transparency;
          endif;
        endif;
        stepplot y=SURVIVAL x=TIME / group=STRATUM index=STRATUMNUM name="Survival"
            rolename=(_tip1=ATRISK _tip2=EVENT) tip=(y x Time _tip1 _tip2) LINEATTRS=(thickness=3px);
        if (PLOTCENSORED)
          scatterplot y=CENSORED x=TIME / group=STRATUM index=STRATUMNUM
          markerattrs=(symbol=CIRCLEFILLED size=8pt);
        endif;
        if (PLOTATRISK)
          innermargin / align=bottom;
            blockplot x=TATRISK block=ATRISK / class=CLASSATRISK repeatedvalues=true display=(
                label values) valuehalign=start valuefitpolicy=truncate labelposition=left
                labelattrs=GRAPHVALUETEXT valueattrs=GRAPHDATATEXT (size=20pt)
                includemissingclass=false;
          endinnermargin;
        endif;
        DiscreteLegend "Survival" / location=inside HALIGN=LEFT VALIGN=BOTTOM border=false;
        if (PLOTCENSORED)
          if (PLOTTEST)
            layout gridded / rows=2 autoalign=(TOPRIGHT BOTTOMLEFT TOP BOTTOM) border=false
                BackgroundColor=GraphWalls:Color Opaque=true;
              entry " Censored";
              if (PVALUE < .0001)
                entry TESTNAME " p " eval (PUT(PVALUE, PVALUE6.4));
              else
                entry TESTNAME " p = " eval (PUT(PVALUE, PVALUE6.4));
              endif;
            endlayout;
          else
            layout gridded / rows=1 autoalign=(TOPRIGHT BOTTOMLEFT TOP BOTTOM) border=false
                BackgroundColor=GraphWalls:Color Opaque=true;
                entry " Censored";
            endlayout;
          endif;
          else
            if (PLOTTEST)
              layout gridded / rows=1 autoalign=(TOPRIGHT BOTTOMLEFT TOP BOTTOM) border=false
                  BackgroundColor=GraphWalls:Color Opaque=true;
              if (PVALUE < .0001)
                entry TESTNAME " p " eval (PUT(PVALUE, PVALUE6.4));
              else
                entry TESTNAME " p = " eval (PUT(PVALUE, PVALUE6.4));
              endif;
              endlayout;
            endif;
          endif;
      endlayout;
    endif;
  EndGraph;
  end;
run;
ods listing gpath = "&Path." style = MyStatistical sge = on;
ods graphics on /
  border = off scale = on
  imagename = "Lifetest_template_ods"
  height = 12 in width = 19 in;
proc lifetest data = D1 plots=(survival(atrisk=(0 to 12.5 by 2.5) test cl));
  time T * Censor(1);
  strata GroupC2;
run;
ods graphics off;
ods listing close; ods listing;
